using System;
using System.Collections.Generic;
using System.Text;
using Utils.Log.Exceptions;

namespace Utils.Log
{
    /// <summary>
    /// A log entry
    /// </summary>
    public class LogRow
    {
        #region Variables
        /// <summary>
        /// Date of the log entry
        /// </summary>
        private DateTime vDate;
        /// <summary>
        /// Type of the log entry
        /// </summary>
        private LogType vType;
        /// <summary>
        /// Title of the log entry
        /// </summary>
        private string vTitle;
        /// <summary>
        /// Message of the log entry
        /// </summary>
        private string vMessage;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public LogRow() { }
        /// <summary>
        /// Constructor (try to parse the _LogEntry
        /// </summary>
        /// <param name="_LogEntry">Log entry that will be parsed</param>
        /// <exception cref="BadLogParseException">Impossible to parse the log</exception>
        public LogRow(string _LogEntry)
        {
            try { Fill(_LogEntry); }
            catch (BadLogParseException) { throw; }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the date of the log entry
        /// </summary>
        public DateTime Date
        {
            get { return vDate; }
            set { vDate = value; }
        }
        /// <summary>
        /// Gets or sets the message of the log entry
        /// </summary>
        public LogType Type
        {
            get { return vType; }
            set { vType = value; }
        }
        /// <summary>
        /// Gets or sets the title of the log entry
        /// </summary>
        public string Title
        {
            get { return vTitle; }
            set { vTitle = value; }
        }
        /// <summary>
        /// Gets or sets the type of the log entry
        /// </summary>
        public string Message
        {
            get { return vMessage; }
            set { vMessage = value; }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Try fill if possible the LogRow with the data of the string _LogEntry
        /// </summary>
        /// <param name="_LogEntry">The log entry to parse</param>
        public bool TryFill(string _LogEntry)
        {
            _LogEntry = _LogEntry.Replace(Log.LineSeparator, "");
            string[] Options = { Log.CellSeparator };
            string[] Cells = _LogEntry.Split(Options, StringSplitOptions.None);

            if (Cells.Length != 4) return false;
            this.Date = DateTime.Parse(Cells[0]);
            #region TypeLog
            if (Cells[1] == "Divers") this.Type = LogType.Divers;
            else if (Cells[1] == "Debug") this.Type = LogType.Debug;
            else if (Cells[1] == "Erreur") this.Type = LogType.Erreur;
            else if (Cells[1] == "Information") this.Type = LogType.Information;
            else if (Cells[1] == "Rapport") this.Type = LogType.Rapport;
            else return false;
            #endregion
            this.Title = Cells[2].TrimEnd(new char[] { ' ' });
            this.Message = Cells[3].TrimEnd(new char[] { ' ' });
            return true;
        }
        /// <summary>
        /// Fill if possible the LogRow with the data of the string _LogEntry
        /// </summary>
        /// <exception cref="BadLogParseException">If the parse failed</exception>
        public void Fill(string _LogEntry)
        {
            _LogEntry = _LogEntry.Replace(Log.LineSeparator, "");
            string[] Options = { Log.CellSeparator };
            string[] Cells = _LogEntry.Split(Options, StringSplitOptions.None);

            if (Cells.Length != 4) throw new BadLogParseException("Too much cells");
            this.Date = DateTime.Parse(Cells[0]);
            #region TypeLog
            if (Cells[1] == "Divers") this.Type = LogType.Divers;
            else if (Cells[1] == "Debug") this.Type = LogType.Debug;
            else if (Cells[1] == "Erreur") this.Type = LogType.Erreur;
            else if (Cells[1] == "Information") this.Type = LogType.Information;
            else if (Cells[1] == "Rapport") this.Type = LogType.Rapport;
            else throw new BadLogParseException("Unknown type");
            #endregion
            this.Title = Cells[2].TrimEnd(new char[] { ' ' });
            this.Message = Cells[3].TrimEnd(new char[] { ' ' });
        }
        /// <summary>
        /// Try to parse the string entry of log and fill if possible the object
        /// </summary>
        /// <param name="_LogRow">LogRow to fill with the string</param>
        /// <param name="_LogEntry">The log entry to parse</param>
        public static bool TryParse(ref LogRow _LogRow, string _LogEntry)
        {
            _LogEntry = _LogEntry.Replace(Log.LineSeparator, "");
            string[] Options = { Log.CellSeparator };
            string[] Cells = _LogEntry.Split(Options, StringSplitOptions.None);

            if (Cells.Length != 4) return false;
            _LogRow.Date = DateTime.Parse(Cells[0]);
            #region TypeLog
            if (Cells[1] == "Divers") _LogRow.Type = LogType.Divers;
            else if (Cells[1] == "Debug") _LogRow.Type = LogType.Debug;
            else if (Cells[1] == "Erreur") _LogRow.Type = LogType.Erreur;
            else if (Cells[1] == "Information") _LogRow.Type = LogType.Information;
            else if (Cells[1] == "Rapport") _LogRow.Type = LogType.Rapport;
            else return false;
            #endregion
            _LogRow.Title = Cells[2].TrimEnd(new char[] { ' ' });
            _LogRow.Message = Cells[3].TrimEnd(new char[] { ' ' });
            return true;
        }
        /// <summary>
        /// Fill the string entry of log and fill if possible the object
        /// </summary>
        /// <param name="_LogEntry">string Log entry</param>
        /// <returns>An instance of LogRow</returns>
        /// <exception cref="BadLogParseException">If the parse failed</exception>
        public static LogRow Parse(string _LogEntry)
        {
            LogRow _LogRow = new LogRow();
            _LogEntry = _LogEntry.Replace(Log.LineSeparator, "");
            string[] Options = { Log.CellSeparator };
            string[] Cells = _LogEntry.Split(Options, StringSplitOptions.RemoveEmptyEntries);

            if (Cells.Length != 4) throw new BadLogParseException("Too much cells");
            _LogRow.Date = DateTime.Parse(Cells[0]);
            #region TypeLog
            Cells[1] = Cells[1].Trim();
            if (Cells[1] == "Divers") _LogRow.Type = LogType.Divers;
            else if (Cells[1] == "Debug") _LogRow.Type = LogType.Debug;
            else if (Cells[1] == "Erreur") _LogRow.Type = LogType.Erreur;
            else if (Cells[1] == "Information") _LogRow.Type = LogType.Information;
            else if (Cells[1] == "Rapport") _LogRow.Type = LogType.Rapport;
            else throw new BadLogParseException("Unknown type");
            #endregion
            _LogRow.Title = Cells[2].TrimEnd(new char[] { ' ' });
            _LogRow.Message = Cells[3].TrimEnd(new char[] { ' ' });
            return _LogRow;
        }
        /// <summary>
        /// Gives the object in string mode
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.Date.ToString() + Log.CellSeparator + this.Type.ToString() + Log.CellSeparator
                + this.Title + Log.CellSeparator + this.Message + Log.CellSeparator + Log.LineSeparator;
        }
        #endregion

        #region Operators
        /// <summary>
        /// Converts the instance of LogRow into a string
        /// </summary>
        /// <param name="_LogRow"></param>
        /// <returns></returns>
        public static implicit operator string(LogRow _LogRow)
        {
            return _LogRow.ToString();
        }
        #endregion
    }
}
